package SortCategories::Tags;

use strict;
use MT::Promise qw( delay );

sub _hdlr_sort_categories {
    my ( $ctx, $args, $cond ) = @_;
    my ( %terms, %args );
    $ctx->set_blog_load_context( $args, \%terms, \%args )
        or return $ctx->error( $ctx->errstr );
    require MT::Placement;
    $args{'sort'} = $args->{sort_by} || 'label';
    $args{'direction'}
        = lc( $args->{sort_order} || '' ) eq 'descend' ? 'descend' : 'ascend';

    my $class_type = $args->{class_type} || 'category';
    my $class = MT->model($class_type);
    my $entry_class
        = MT->model( $class_type eq 'category' ? 'entry' : 'page' );
    my %counts;
    my $count_tag
        = $class_type eq 'category'
        ? 'CategoryCount'
        : 'FolderCount';
    my $uncompiled = $ctx->stash('uncompiled') || '';
    my $count_all = 0;

    if ( !$args->{show_empty} || $uncompiled =~ /<\$?mt:?$count_tag/i ) {
        $count_all = 1;
    }

    ## Supplies a routine that will yield the number of entries associated
    ## with the category in context in the most efficient manner.
    ## If we can determine counts will be gathered for all categories,
    ## a 'count_group_by' request is done for MT::Placement to fetch counts
    ## with a single query (storing them in %counts).
    ## Otherwise, counts are collected on an as-needed basis, using the
    ## 'entry_count' method in MT::Category.
    my $counts_fetched = 0;
    my $entry_count_of = sub {
        my $cat = shift;
        return delay( sub { $cat->entry_count } )
            unless $count_all;
        return $cat->entry_count(
            defined $counts{ $cat->id } ? $counts{ $cat->id } : 0 )
            if $counts_fetched;
        return $cat->cache_property(
            'entry_count',
            sub {

                # issue a single count_group_by for all categories
                my $cnt_iter = MT::Placement->count_group_by(
                    {%terms},
                    {   group => ['category_id'],
                        join  => $entry_class->join_on(
                            undef,
                            {   id     => \'=placement_entry_id',
                                status => MT::Entry::RELEASE(),
                            }
                        ),
                    }
                );
                while ( my ( $count, $cat_id ) = $cnt_iter->() ) {
                    $counts{$cat_id} = $count;
                }
                $counts_fetched = 1;
                $counts{ $cat->id };
            }
        );
    };

    my $iter    = $class->load_iter( \%terms, \%args );
    my $res     = '';
    my $builder = $ctx->stash('builder');
    my $tokens  = $ctx->stash('tokens');
    my $glue    = $args->{glue};
    ## In order for this handler to double as the handler for
    ## <MTArchiveList archive_type="Category">, it needs to support
    ## the <$MTArchiveLink$> and <$MTArchiveTitle$> tags
    local $ctx->{inside_mt_categories} = 1;
    my $i   = 0;
    my $cat = $iter->();
    if ( !$args->{show_empty} ) {
        while ( defined $cat && !$entry_count_of->($cat) ) {
            $cat = $iter->();
        }
    }
    my $n = $args->{lastn};
    my $vars = $ctx->{__stash}{vars} ||= {};
    while ( defined($cat) ) {
        $i++;
        my $next_cat = $iter->();
        if ( !$args->{show_empty} ) {
            while ( defined $next_cat && !$entry_count_of->($next_cat) ) {
                $next_cat = $iter->();
            }
        }
        my $last;
        $last = 1 if $n && ( $i >= $n );
        $last = 1 unless defined $next_cat;

        local $ctx->{__stash}{category} = $cat;
        local $ctx->{__stash}{entries};
        local $ctx->{__stash}{category_count};
        local $ctx->{__stash}{blog_id} = $cat->blog_id;
        local $ctx->{__stash}{blog}    = MT::Blog->load( $cat->blog_id );
        local $ctx->{__stash}{folder_header} = ( $i == 1 )
            if $class_type ne 'category';
        local $ctx->{__stash}{folder_footer} = ($last)
            if $class_type ne 'category';
        local $vars->{__first__}   = $i == 1;
        local $vars->{__last__}    = $last;
        local $vars->{__odd__}     = ( $i % 2 ) == 1;
        local $vars->{__even__}    = ( $i % 2 ) == 0;
        local $vars->{__counter__} = $i;
        $ctx->{__stash}{category_count} = $entry_count_of->($cat);
        defined(
            my $out = $builder->build(
                $ctx, $tokens,
                {   %$cond,
                    ArchiveListHeader => $i == 1,
                    ArchiveListFooter => $last
                }
            )
        ) or return $ctx->error( $builder->errstr );
        $res .= $glue if defined $glue && length($res) && length($out);
        $res .= $out;
        last if $last;
        $cat = $next_cat;
    }
    $res;
}

1;
